/****************************************************************************
** mainwin_p.h - classes used privately by the main window.
** Copyright (C) 2001, 2002  Justin Karneges, Michail Pishchagin
**
** This program is free software; you can redistribute it and/or
** modify it under the terms of the GNU General Public License
** as published by the Free Software Foundation; either version 2
** of the License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
**
****************************************************************************/

#ifndef MAINWIN_P_H

#include<qpushbutton.h>
#include<qtoolbutton.h>
#include<qpopupmenu.h>
#include<qlayout.h>
#include<qlabel.h>
#include<qptrvector.h>
#include<qtooltip.h>
#include<qaction.h>
#include"trayicon.h"
#include"iconset.h"
#include"psicon.h"

class MPushButton : public AnimatedButton
{
	Q_OBJECT
public:
	MPushButton(QWidget *parent=0, const char *name=0);

	void setText(const QString &);

protected:
	// reimplemented
	void drawButtonLabel(QPainter *);

	void paintEvent(QPaintEvent*);
	QSize minimumSizeHint() const;
	QSize sizeHint() const;

private:
	bool hasTip;

	friend class PopupAction;
};

class PopupAction : public QAction
{
	Q_OBJECT

private:
	class Private;
	Private *d;

private slots:
	void objectDestroyed ();

public slots:
	void setEnabled (bool);

public:
	PopupAction (const QString &label, QPopupMenu *_menu, QObject *parent, const char *name);
	void setSizePolicy (const QSizePolicy &p);
	void setAnim (Anim *a);
	void setIconSet (const QIconSet &icon);
	void setPixmap (const QPixmap &pix);
	void setText (const QString &text);
	bool addTo (QWidget *w);
};

class MLabel : public QLabel
{
	Q_OBJECT
public:
	MLabel(QWidget *parent=0, const char *name=0);

protected:
	// reimplemented
	void mouseReleaseEvent(QMouseEvent *);
	void mouseDoubleClickEvent(QMouseEvent *);

signals:
	void clicked(int);
	void doubleClicked();
};

class MTray : public QObject
{
	Q_OBJECT
public:
	MTray(const QImage &icon, const QString &tip, QPopupMenu *popup, QObject *parent=0);
	~MTray();

	void setToolTip(const QString &);
	void setImage(const QImage &);
	void setAnim(Anim *);
	bool isAnimating() const;

signals:
	void clicked(const QPoint &, int);
	void doubleClicked(const QPoint &);
	void closed();

public slots:
	void show();
	void hide();

private slots:
	void animate();

private:
	TrayIcon *ti;
	QTimer *trayTimer;
	Anim *anim;
	int animStep;

	QPixmap makeIcon(const QImage &_in);
};

class MAction : public QAction
{
	Q_OBJECT

private:
	class Private;
	Private *d;

	void init (const QIconSet &i, int id, PsiCon *psi);

private slots:
	void numAccountsChanged();
	void itemActivated(int n);
	void objectDestroyed();

	void setEnabled (bool);

public slots:
	void setIconSet (const QIconSet &icon);

public:
	MAction(const QIconSet &, const QString &, int id, PsiCon *, QObject *parent=0);
	MAction(const QString &, int id, PsiCon *, QObject *parent=0);

	bool addTo(QWidget *w);

signals:
	void activated(PsiAccount *, int);
};

#endif

