/****************************************************************************
** optionsdlg.cpp - the preferences dialog
** Copyright (C) 2001, 2002  Justin Karneges
**
** This program is free software; you can redistribute it and/or
** modify it under the terms of the GNU General Public License
** as published by the Free Software Foundation; either version 2
** of the License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
**
****************************************************************************/

#include"optionsdlg.h"
#include"common.h"
#include"iconset.h"

#include<qcolordialog.h>
#include<qfontdialog.h>
#include<qcheckbox.h>
#include<qcombobox.h>
#include<qlineedit.h>
#include<qlistbox.h>
#include<qinputdialog.h>
#include<qspinbox.h>
#include<qfiledialog.h>
#include<qtextedit.h>
#include<qdir.h>
#include<qlabel.h>
#include<qwhatsthis.h>
#include<qtoolbutton.h>
#include<qtimer.h>
#include"psicon.h"
//#include"openpgp.h"

#include"eventdlg.h"
#include"chatdlg.h"

int om_x11browse[] = { 0, 2, 1 };

static QString lastPath;

OptionsDlg::OptionsDlg(PsiCon *_psi, const Options &_opt, QWidget *parent, const char *name)
:OptionsUI(parent, name, FALSE, WDestructiveClose)
{
	psi = _psi;
	psi->dialogRegister(this);

	int n;

	setCaption(CAP(tr("Options")));

	opt = _opt;

	// General : Roster
	ck_useleft->setChecked(opt.useleft);
	ck_singleclick->setChecked(opt.singleclick);

	bg_defAct = new QButtonGroup;
	bg_defAct->setRadioButtonExclusive(TRUE);
	bg_defAct->insert(rb_defActMsg, 0);
	bg_defAct->insert(rb_defActChat, 1);
	bg_defAct->setButton(opt.defaultAction);

	ck_showself->setChecked(opt.alwaysShowSelf);

	// General : Misc
#ifdef Q_WS_MAC
	ck_alwaysOnTop->setEnabled(FALSE);
#else
	ck_alwaysOnTop->setChecked(opt.alwaysOnTop);
#endif

	bg_delChats = new QButtonGroup;
	bg_delChats->setRadioButtonExclusive(TRUE);
	bg_delChats->insert(rb_delChatsClose, 0);
	bg_delChats->insert(rb_delChatsHour, 1);
	bg_delChats->insert(rb_delChatsDay, 2);
	bg_delChats->insert(rb_delChatsNever, 3);
	bg_delChats->setButton(opt.delChats);

	ck_keepSizes->setChecked(opt.keepSizes);
	ck_useEmoticons->setChecked(opt.useEmoticons);
	ck_chatSays->setChecked(opt.chatSays);

	ck_jidComplete->setChecked(opt.jidComplete);
	ck_grabUrls->setChecked(opt.grabUrls);

	// General : Links
	connect(cb_link, SIGNAL(activated(int)), this, SLOT(selectBrowser(int)));
#ifdef Q_WS_WIN
	cb_link->insertItem(tr("Windows Default Browser/Mail"));
	cb_link->insertItem(tr("Custom"));
	cb_link->setCurrentItem(opt.browser);
	selectBrowser(opt.browser);
#endif
#ifdef Q_WS_X11
	cb_link->insertItem(tr("KDE Default Browser/Mail"));
	cb_link->insertItem(tr("GNOME2 Default Browser/Mail"));
	cb_link->insertItem(tr("Custom"));
	int rbi = om_x11browse[opt.browser];
	cb_link->setCurrentItem(rbi);
	selectBrowser(rbi);
#endif
#ifdef Q_WS_MAC
	cb_link->insertItem(tr("MacOS Default Browser/Mail"));
	cb_link->setCurrentItem(opt.browser);
	selectBrowser(opt.browser);
#endif
	le_linkBrowser->setText(opt.customBrowser);
	le_linkMailer->setText(opt.customMailer);

	// General : Docking
	ck_docklet->setChecked(opt.useDock);
	ck_dockDCstyle->setChecked(opt.dockDCstyle);
	ck_dockHideMW->setChecked(opt.dockHideMW);
	ck_dockToolMW->setChecked(opt.dockToolMW);
	connect(ck_docklet, SIGNAL(toggled(bool)), ck_dockDCstyle, SLOT(setEnabled(bool)));
	connect(ck_docklet, SIGNAL(toggled(bool)), ck_dockHideMW, SLOT(setEnabled(bool)));
	connect(ck_docklet, SIGNAL(toggled(bool)), ck_dockToolMW, SLOT(setEnabled(bool)));
	if(!ck_docklet->isChecked()) {
		ck_dockDCstyle->setEnabled(FALSE);
		ck_dockHideMW->setEnabled(FALSE);
		ck_dockToolMW->setEnabled(FALSE);
	}

#ifdef Q_WS_MAC
	gb_docking->setEnabled(FALSE);
#endif

	// Look and Feel : Colors
	bg_color = new QButtonGroup;
	bg_color->insert(pb_cOnline, 0);
	bg_color->insert(pb_cOffline, 1);
	bg_color->insert(pb_cAway, 2);
	bg_color->insert(pb_cDND, 3);
	bg_color->insert(pb_cGroupFore, 4);
	bg_color->insert(pb_cGroupBack, 5);
	bg_color->insert(pb_cListBack, 6);
	for(n = 0; n < 7; ++n) {
		showColor((QPushButton *)bg_color->find(n), opt.color[n]);
	}
	connect(bg_color, SIGNAL(clicked(int)), this, SLOT(chooseColor(int)));

	// Look and Feel : Fonts
	le_font[0] = le_fRoster;
	le_font[1] = le_fMessage;
	le_font[2] = le_fChat;
	for(n = 0; n < 3; ++n) {
		le_font[n]->setText(opt.font[n]);
	}
	bg_font = new QButtonGroup;
	bg_font->insert(pb_fRoster, 0);
	bg_font->insert(pb_fMessage, 1);
	bg_font->insert(pb_fChat, 2);
	connect(bg_font, SIGNAL(clicked(int)), this, SLOT(chooseFont(int)));

	// Look and Feel : Icon Set
	QStringList descriptions;
	PsiIconSet::getIconSetsList(iconsets, descriptions);
	cb_iconset->insertStringList(descriptions);
	n = qstringlistmatch(iconsets, opt.iconset);
	cb_iconset->setCurrentItem(n);
	connect(cb_iconset, SIGNAL(activated(int)), this, SLOT(selectIconSet(int)));

	// Security : OpenPGP
	//QTimer::singleShot(0, this, SLOT(buildPGPList()));

	// Events : Receive
	ck_popupMsgs->setChecked(opt.popupMsgs);
	ck_popupChats->setChecked(opt.popupChats);
	ck_allowAwayPopup->setChecked(!opt.noAwayPopup);
	ck_allowUnlistedPopup->setChecked(!opt.noUnlistedPopup);
	ck_raise->setChecked(opt.raise);
	cb_incomingAs->setCurrentItem(opt.incomingAs);

	// Events : Alerts
	bg_alerts->setButton(opt.alertStyle);
	connect(bg_alerts, SIGNAL(clicked(int)), this, SLOT(selectAlertStyle(int)));

	// Events : Misc
	ck_ignoreHeadline->setChecked(opt.ignoreHeadline);
	ck_scrollTo->setChecked(opt.scrollTo);
	ck_alertOpenChats->setChecked(opt.alertOpenChats);

	// Presence : Misc
	ck_askOnline->setChecked(opt.askOnline);
	ck_rosterAnim->setChecked(opt.rosterAnim);
	ck_autoVersion->setChecked(opt.autoVersion);

	// Presence : Auto Status
	sb_asAway->setMinValue(0);
	sb_asAway->setValue(opt.asAway);
	sb_asXa->setMinValue(0);
	sb_asXa->setValue(opt.asXa);
	sb_asOffline->setMinValue(0);
	sb_asOffline->setValue(opt.asOffline);
	if(opt.asAway <= 0)
		opt.use_asAway = FALSE;
	if(opt.asXa <= 0)
		opt.use_asXa = FALSE;
	if(opt.asOffline <= 0)
		opt.use_asOffline = FALSE;
	ck_asAway->setChecked(opt.use_asAway);
	ck_asXa->setChecked(opt.use_asXa);
	ck_asOffline->setChecked(opt.use_asOffline);
	te_asMessage->setText(opt.asMessage);

	// Presence : Status Presets
	pb_spNew->setEnabled(TRUE);
	pb_spDelete->setEnabled(FALSE);
	te_sp->setEnabled(FALSE);
	connect(pb_spNew, SIGNAL(clicked()), this, SLOT(newStatusPreset()));
	connect(pb_spDelete, SIGNAL(clicked()), this, SLOT(removeStatusPreset()));
	connect(te_sp, SIGNAL(textChanged()), this, SLOT(changeStatusPreset()));
	connect(lb_sp, SIGNAL(highlighted(int)), this, SLOT(selectStatusPreset(int)));
	lb_sp->insertStringList(opt.sp.varsToStringList());
	if(lb_sp->count() >= 1)
		lb_sp->setSelected(0, TRUE);

	// Sound : Preferences
#if defined(Q_WS_WIN)
	lb_player->setEnabled(FALSE);
	le_player->setText(tr("Windows Sound"));
	le_player->setEnabled(FALSE);
#elif defined(Q_WS_MAC)
	lb_player->setEnabled(FALSE);
	le_player->setText(tr("MacOS Sound"));
	le_player->setEnabled(FALSE);
#else
	le_player->setText(opt.player);
#endif
	ck_awaySound->setChecked(!opt.noAwaySound);
	ck_gcSound->setChecked(!opt.noGCSound);

	// Sound : Events
	le_oe[0] = le_oeMessage;
	le_oe[1] = le_oeChat1;
	le_oe[2] = le_oeChat2;
	le_oe[3] = le_oeSystem;
	le_oe[4] = le_oeOnline;
	le_oe[5] = le_oeOffline;
	le_oe[6] = le_oeSend;
	for(n = 0; n < 7; ++n)
		le_oe[n]->setText(opt.onevent[n]);

	bg_se = new QButtonGroup;
	bg_se->insert(tb_seMessage, 0);
	bg_se->insert(tb_seChat1, 1);
	bg_se->insert(tb_seChat2, 2);
	bg_se->insert(tb_seSystem, 3);
	bg_se->insert(tb_seOnline, 4);
	bg_se->insert(tb_seOffline, 5);
	bg_se->insert(tb_seSend, 6);
	connect(bg_se, SIGNAL(clicked(int)), this, SLOT(chooseSoundEvent(int)));

	bg_sePlay = new QButtonGroup;
	bg_sePlay->insert(tb_seMessagePlay, 0);
	bg_sePlay->insert(tb_seChat1Play, 1);
	bg_sePlay->insert(tb_seChat2Play, 2);
	bg_sePlay->insert(tb_seSystemPlay, 3);
	bg_sePlay->insert(tb_seOnlinePlay, 4);
	bg_sePlay->insert(tb_seOfflinePlay, 5);
	bg_sePlay->insert(tb_seSendPlay, 6);
	connect(bg_sePlay, SIGNAL(clicked(int)), this, SLOT(previewSoundEvent(int)));

	connect(pb_soundReset, SIGNAL(clicked()), this, SLOT(soundReset()));

	//Group Chat
	connect(pb_addHighlightWord, SIGNAL(clicked()), this, SLOT(addGCHighlight()));
	connect(pb_removeHighlightWord, SIGNAL(clicked()), this, SLOT(removeGCHighlight()));
	connect(pb_addNickColor, SIGNAL(clicked()), this, SLOT(addGCNickColor()));
	connect(pb_removeNickColor, SIGNAL(clicked()), this, SLOT(removeGCNickColor()));
	connect(le_newNickColor, SIGNAL(textChanged(const QString &)), this, SLOT(displayGCNickColor()));
	connect(pb_nickColor, SIGNAL(clicked()), this, SLOT(chooseGCNickColor()));
	connect(lb_nickColors, SIGNAL(highlighted(int)), this, SLOT(selectedGCNickColor()));
	ck_gcHighlights->setChecked(true);
	ck_gcHighlights->setChecked(opt.gcHighlighting);
	ck_gcNickColoring->setChecked(true);
	ck_gcNickColoring->setChecked(opt.gcNickColoring);
	lb_highlightWords->insertStringList(opt.gcHighlights);
	lb_nickColors->insertStringList(opt.gcNickColors);

	connect(buttonApply, SIGNAL(clicked()), this, SLOT(apply()));
	buttonApply->setEnabled(FALSE);
	dirty = FALSE;
	noDirty = FALSE;

	// QWhatsThis helpers
	QString s;
	QWhatsThis::add(ck_useleft,
		tr("Normally, right-clicking with the mouse on a contact will activate the context-menu."
		"  Check this option if you'd rather use a left-click."));
	QWhatsThis::add(ck_singleclick,
		tr("Normally, a double-click on a contact will invoke the default action."
		"  Check this option if you'd rather invoke with a single-click."));
	QWhatsThis::add(rb_defActMsg,
		tr("Make the default action open a chat window."));
	QWhatsThis::add(rb_defActChat,
		tr("Make the default action open a normal message window."));
	QWhatsThis::add(ck_alwaysOnTop,
		tr("Makes the main Psi window always be in front of other windows."));

	s = tr("<P>Controls how long the chat log will be kept in memory after the"
		" chat window is closed.</P>");
	QWhatsThis::add(rb_delChatsClose, s +
		tr("<P>This option does not keep the chat log in memory.</P>"));
	QWhatsThis::add(rb_delChatsHour, s +
		tr("<P>This option keeps the chat log for 1 hour before deleting it.</P>"));
	QWhatsThis::add(rb_delChatsDay, s +
		tr("<P>This option keeps the chat log for 1 day before deleting it.</P>"));
	QWhatsThis::add(rb_delChatsNever, s +
		tr("<P>This options keeps the chat log forever.</P>"));
	QWhatsThis::add(ck_keepSizes,
		tr("Makes Psi remember window size and positions for chats and messages."
		"  If disabled, the windows will always appear in their default positions and sizes."));
	QWhatsThis::add(ck_useEmoticons,
		tr("<P>Emoticons are short sequences of characters that are used to convey an emotion or idea.</P>"
		"<P>Enable this option if you want Psi to replace common emoticons with a graphical image.</P>"
		"<P>For example, <B>:-)</B> would be relaced by <img src=\"smile\"></P>"));
	QWhatsThis::add(ck_chatSays,
		tr("<P>Changes the normal chat style from:</P>"
		"<P>[01:23:45] <MyName> Hi</P>"
		"<P>[01:23:56] <YourName> How are you?</P>"
		"<P>to:</P>"
		"<P>[01:23:45] MyName says:</P>"
		"<P>Hi</P>"
		"<P>[01:23:56] YourName says:</P>"
		"<P>How are you?</P>"));
	QWhatsThis::add(cb_link,
		tr("Selects what applications to use for handling URLs and e-mail addresses."
		"  You can choose between the system default and custom applications."));

	s = tr("Enter the path to the application's executable and choose \"Custom\" in the list above.");
	QWhatsThis::add(le_linkBrowser,
		tr("Specify what custom browser application to use for handling URLs here.") + "  " + s);
	QWhatsThis::add(le_linkMailer,
		tr("Specify what custom mailer application to use for handling e-mail addresses here.") + "  " + s);
	QWhatsThis::add(ck_docklet,
		tr("Makes Psi use a docklet icon, also known as system tray icon."));
	QWhatsThis::add(ck_dockDCstyle,
		tr("Normally, single-clicking on the Psi docklet icon brings the main window to"
		" the foreground.  Check this option if you would rather use a double-click."));
	QWhatsThis::add(ck_dockHideMW,
		tr("Starts Psi with only the docklet icon visible."));
	QWhatsThis::add(ck_dockToolMW,
		tr("Prevents Psi from taking up a slot on the taskbar and makes the main "
		"window use a small titlebar."));

	s = tr("Specifies the text color for a contact name in the main window when that user is \"%1\".");
	QWhatsThis::add(pb_cOnline,
		s.arg(tr("online")));
	QWhatsThis::add(pb_cOffline,
		s.arg(tr("offline")));
	QWhatsThis::add(pb_cAway,
		s.arg(tr("away")));
	QWhatsThis::add(pb_cDND,
		s.arg(tr("do not disturb")));
	QWhatsThis::add(pb_cGroupBack,
		tr("Specifies the background color for a group name in the main window."));
	QWhatsThis::add(pb_cListBack,
		tr("Specifies the background color for the main window."));
	QWhatsThis::add(le_font[0],
		tr("Specifies the font style for the main window."));
	QWhatsThis::add(le_font[1],
		tr("Specifies the font style for message windows."));
	QWhatsThis::add(le_font[2],
		tr("Specifies the font style for chat windows."));
	QWhatsThis::add(pb_fRoster,
		tr("Selects a font for the a roster window using the font selection dialog."));
	QWhatsThis::add(pb_fMessage,
		tr("Selects a font for message windows using the font selection dialog."));
	QWhatsThis::add(pb_fChat,
		tr("Selects a font for chat windows using the font selection dialog."));
	QWhatsThis::add(cb_iconset,
		tr("Selects a set of icons to use in the main window."
		"  Icons are usually used for visual identification of users'"
		" status, such as \"online\", \"offline\", \"away\", etc."));
	QWhatsThis::add(ck_popupMsgs,
		tr("Makes new incoming message windows pop up automatically when received."));
	QWhatsThis::add(ck_popupChats,
		tr("Makes new incoming chat windows pop up automatically when received."));
	QWhatsThis::add(ck_allowAwayPopup,
		tr("Normally, Psi will not autopopup events when you are away.  "
		"Set this option if you want them to popup anyway."));
	QWhatsThis::add(ck_allowUnlistedPopup,
		tr("Normally, Psi will not autopopup events from users not in your roster.  "
		"Set this option if you want them to popup anyway."));
	QWhatsThis::add(ck_raise,
		tr("Makes new incoming events bring the main window to the foreground."));
	QWhatsThis::add(cb_incomingAs,
		tr("<P>Specifies how to treat incoming events:</P>"
		"<P><B>Normal</B> - messages come as messages, chats come as chats.</P>"
		"<P><B>Messages</B> - All messages/chats come as messages, no matter what their original form was.</P>"
		"<P><B>Chats</B> - All messages/chats come as chats, no matter what their original form was.</P>"));
	QWhatsThis::add(rb_aSolid,
		tr("Does not animate or blink incoming event icons on the main window as they are received."));
	QWhatsThis::add(rb_aBlink,
		tr("Makes all incoming event icons blink on the main window as events are received."));
	QWhatsThis::add(rb_aAnimate,
		tr("Animates incoming event icons on the main window as events are recieved."));
	QWhatsThis::add(ck_ignoreHeadline,
		tr("Makes Psi ignore all incoming \"headline\" events,"
		" like system-wide news on MSN, announcements, etc."));
	QWhatsThis::add(ck_scrollTo,
		tr("Makes Psi scroll the main window automatically so you can see new incoming events."));
	QWhatsThis::add(ck_alertOpenChats,
		tr("Normally, Psi will not alert you when a new chat message"
		" is received in a chat window that is already open."
		"  Check this option if you want to receive these alerts anyway."));
	QWhatsThis::add(ck_askOnline,
		tr("Jabber allows you to put extended status messages on"
		" all status types.  Normally, Psi does not prompt you for"
		" an extended message when you set your status to \"online\"."
		"  Check this option if you want to have this prompt."));
	QWhatsThis::add(ck_rosterAnim,
		tr("Makes Psi animate contact names in the main window when they come online."));

	s = tr("Makes Psi automatically set your status to \"away\" if your"
		" computer is idle for the specified amount of time.");
	QWhatsThis::add(ck_asAway, s);
	QWhatsThis::add(sb_asAway, s);
	s = tr("Makes Psi automatically set your status to \"extended away\" if your"
		" computer is idle for the specified amount of time.");
	QWhatsThis::add(ck_asXa, s);
	QWhatsThis::add(sb_asXa, s);
	s = tr("Makes Psi automatically set your status to \"offline\" if your"
		" computer is idle for the specified amount of time."
		"  This will disconnect you from the Jabber server.");
	QWhatsThis::add(ck_asOffline, s);
	QWhatsThis::add(sb_asOffline, s);

	QWhatsThis::add(te_asMessage,
		tr("Specifies an extended message to use if you allow Psi"
		" to set your status automatically.  See options above."));
	QWhatsThis::add(pb_spNew,
		tr("Press this button to create a new status message preset."));
	QWhatsThis::add(pb_spDelete,
		tr("Press this button to delete a status message preset."));
	QWhatsThis::add(lb_sp,
		tr("Use this list to select a status message preset"
		" to view or edit in the box to the right."));
	QWhatsThis::add(te_sp,
		tr("You may edit the message here for the currently selected"
		" status message preset in the list to the left."));
	QWhatsThis::add(le_player,
		tr("If your system supports multiple sound players, you may"
		" choose your preferred sound player application here."));
	QWhatsThis::add(ck_awaySound,
		tr("Enable this option if you wish to hear sound alerts when your status is \"away\" or \"extended away\"."));

	resize(minimumSize());
}

OptionsDlg::~OptionsDlg()
{
	delete bg_defAct;
	delete bg_color;
	delete bg_font;
	delete bg_se;
	delete bg_sePlay;

	psi->dialogUnregister(this);
}

void OptionsDlg::accept()
{
	doApplyOptions();

	QDialog::accept();
}

void OptionsDlg::doApplyOptions()
{
	opt.useleft = ck_useleft->isChecked();
	opt.singleclick = ck_singleclick->isChecked();
	opt.defaultAction = bg_defAct->id(bg_defAct->selected());
	opt.alwaysShowSelf = ck_showself->isChecked();
	opt.delChats = bg_delChats->id(bg_delChats->selected());
#ifdef Q_WS_X11
	opt.browser = om_x11browse[cb_link->currentItem()];
#else
	opt.browser = cb_link->currentItem();
#endif

	opt.alwaysOnTop = ck_alwaysOnTop->isChecked();
	opt.rosterAnim = ck_rosterAnim->isChecked();
	opt.autoVersion = ck_autoVersion->isChecked();

	opt.useDock = ck_docklet->isChecked();
	opt.dockDCstyle = ck_dockDCstyle->isChecked();
	opt.dockHideMW = ck_dockHideMW->isChecked();
	opt.dockToolMW = ck_dockToolMW->isChecked();

	opt.customBrowser = le_linkBrowser->text();
	opt.customMailer = le_linkMailer->text();

	opt.popupMsgs = ck_popupMsgs->isChecked();
	opt.popupChats = ck_popupChats->isChecked();
	opt.noAwayPopup = !ck_allowAwayPopup->isChecked();
	opt.noUnlistedPopup = !ck_allowUnlistedPopup->isChecked();
	opt.raise = ck_raise->isChecked();
	opt.incomingAs = cb_incomingAs->currentItem();

	opt.askOnline = ck_askOnline->isChecked();

	opt.asAway = sb_asAway->value();
	opt.asXa = sb_asXa->value();
	opt.asOffline = sb_asOffline->value();
	opt.use_asAway = ck_asAway->isChecked();
	opt.use_asXa = ck_asXa->isChecked();
	opt.use_asOffline = ck_asOffline->isChecked();
	opt.asMessage = te_asMessage->text();

	opt.player = le_player->text();
	opt.noAwaySound = !ck_awaySound->isChecked();
	opt.noGCSound = !ck_gcSound->isChecked();

	opt.keepSizes = ck_keepSizes->isChecked();
	opt.ignoreHeadline = ck_ignoreHeadline->isChecked();
	opt.scrollTo = ck_scrollTo->isChecked();
	opt.useEmoticons = ck_useEmoticons->isChecked();
	opt.alertOpenChats = ck_alertOpenChats->isChecked();
	opt.chatSays = ck_chatSays->isChecked();

	opt.jidComplete = ck_jidComplete->isChecked();
	opt.grabUrls = ck_grabUrls->isChecked();

	// restore the default sizes
	if(!opt.keepSizes) {
		opt.sizeEventDlg = EventDlg::defaultSize();
		opt.sizeChatDlg = ChatDlg::defaultSize();
	}

	for(int n = 0; n < 7; ++n)
		opt.onevent[n] = le_oe[n]->text();

	// pgp
	/*int x = cb_pgpengine->currentItem();
	if(x == 0)
		opt.pgp = "";
	else
		opt.pgp = pgpEngines[x-1];*/

	//groupchat
	opt.gcHighlighting=ck_gcHighlights->isChecked();
	opt.gcNickColoring=ck_gcNickColoring->isChecked();
	lb_highlightWords->clear();
	lb_highlightWords->insertStringList(opt.gcHighlights);
       
	applyOptions(opt);
}

void OptionsDlg::showColor(QPushButton *pb, QColor c)
{
	QPixmap p(22, 10);
	p.fill(c);
	pb->setPixmap(p);
}

void OptionsDlg::chooseFont(int x)
{
	bool ok;
	QFont font;
	font.fromString(opt.font[x]);

	opt.font[x] = QFontDialog::getFont(&ok, font, this).toString();
	le_font[x]->setText(opt.font[x]);

	if(ok)
		dataChanged();
}

void OptionsDlg::chooseColor(int x)
{
	QColor c = opt.color[x];

	c = QColorDialog::getColor(c, this);
	if(c.isValid()) {
		opt.color[x] = c;
		showColor((QPushButton *)bg_color->find(x), opt.color[x]);

		dataChanged();
	}
}

void OptionsDlg::selectIconSet(int x)
{
	opt.iconset = qstringlistlookup(iconsets, x);
}

void OptionsDlg::selectAlertStyle(int x)
{
	opt.alertStyle = x;
}

void OptionsDlg::selectStatusPreset(int x)
{
	if(x == -1) {
		pb_spDelete->setEnabled(FALSE);
		noDirty = TRUE;
		te_sp->setText("");
		te_sp->setEnabled(FALSE);
		noDirty = FALSE;
		return;
	}

	noDirty = TRUE;

	pb_spDelete->setEnabled(TRUE);

	te_sp->setText(opt.sp.get(lb_sp->text(x)));
	te_sp->setEnabled(TRUE);

	noDirty = FALSE;
}

void OptionsDlg::newStatusPreset()
{
	QString text;

	while(1) {
		bool ok = FALSE;
		text = QInputDialog::getText(
			CAP(tr("New Status Preset")),
			tr("Please enter a name for the new status preset:"),
			QLineEdit::Normal, text, &ok, this);
		if(!ok)
			return;

		if(text.isEmpty())
			QMessageBox::information(this, tr("Error"), tr("Can't create a blank preset!"));
		else if(opt.sp.findByKey(text) != opt.sp.end())
			QMessageBox::information(this, tr("Error"), tr("You already have a preset with that name!"));
		else
			break;
	}

	opt.sp.set(text, "");
	lb_sp->insertItem(text);
	lb_sp->setSelected(lb_sp->count()-1, TRUE);
	te_sp->setFocus();

	dataChanged();
}

void OptionsDlg::removeStatusPreset()
{
	int id = lb_sp->currentItem();
	if(id == -1)
		return;

	dataChanged();

	opt.sp.unset(lb_sp->text(id));
	lb_sp->removeItem(id);

	// select a new entry if possible
	if(lb_sp->count() == 0) {
		selectStatusPreset(-1);
		return;
	}

	if(id >= (int)lb_sp->count())
		id = lb_sp->count()-1;

	lb_sp->setSelected(id, TRUE);
	selectStatusPreset(id);
}

void OptionsDlg::changeStatusPreset()
{
	int id = lb_sp->currentItem();
	if(id == -1)
		return;

	opt.sp.set(lb_sp->text(id), te_sp->text());
	dataChanged();
}

void OptionsDlg::chooseSoundEvent(int x)
{
	if(lastPath.isEmpty())
		lastPath = QDir::homeDirPath();
	QString str = QFileDialog::getOpenFileName(lastPath, tr("Sound (*.wav)"), this, "", tr("Choose a sound file"));
	if(!str.isEmpty()) {
		QFileInfo fi(str);
		lastPath = fi.dirPath();
		le_oe[x]->setText(str);
		dataChanged();
	}
}

void OptionsDlg::previewSoundEvent(int x)
{
	soundPlay(le_oe[x]->text());
}

void OptionsDlg::soundReset()
{
	opt.onevent[eMessage] = g.pathBase + "/sound/chat2.wav";
	opt.onevent[eChat1]   = g.pathBase + "/sound/chat1.wav";
	opt.onevent[eChat2]   = g.pathBase + "/sound/chat2.wav";
	opt.onevent[eSystem]  = g.pathBase + "/sound/chat2.wav";
	opt.onevent[eOnline]  = g.pathBase + "/sound/online.wav";
	opt.onevent[eOffline] = g.pathBase + "/sound/offline.wav";
	opt.onevent[eSend]    = g.pathBase + "/sound/send.wav";

	for(int n = 0; n < 7; ++n)
		le_oe[n]->setText(opt.onevent[n]);

	dataChanged();
}

void OptionsDlg::dataChanged()
{
	if(noDirty)
		return;

	dirty = TRUE;
	buttonApply->setEnabled(TRUE);
}

void OptionsDlg::apply()
{
	doApplyOptions();

	dirty = FALSE;
	buttonApply->setEnabled(FALSE);
}

void OptionsDlg::selectBrowser(int x)
{
	bool enableCustom = TRUE;

#ifdef Q_WS_WIN
	if(x == 0)
		enableCustom = FALSE;
#endif
#ifdef Q_WS_X11
	if(x == 0 || x == 1)
		enableCustom = FALSE;
#endif
#ifdef Q_WS_MAC
	if(x == 0)
		enableCustom = FALSE;
#endif

	gb_linkCustom->setEnabled(enableCustom);
}

/*void OptionsDlg::buildPGPList()
{
	pgpEngines.clear();
	QStringList descriptions;
	descriptions += "None";
	QValueList<OpenPGP::EngineItem> list = OpenPGP::getAvailableEngines();
	int x = -1;
	int n = 0;
	for(QValueList<OpenPGP::EngineItem>::ConstIterator it = list.begin(); it != list.end(); ++it) {
		const OpenPGP::EngineItem i = *it;
		pgpEngines.append(i.first);
		descriptions.append(i.second);
		if(i.first == opt.pgp)
			x = n;
		++n;
	}
	cb_pgpengine->insertStringList(descriptions);

	if(x != -1)
		cb_pgpengine->setCurrentItem(x+1);
	else
		cb_pgpengine->setCurrentItem(0);
}*/

void OptionsDlg::addGCHighlight()
{
	if (le_newHighlightWord->text().isEmpty())
		return;
	opt.gcHighlights<<le_newHighlightWord->text();
	lb_highlightWords->insertItem(le_newHighlightWord->text());
	le_newHighlightWord->setFocus();
	le_newHighlightWord->setText("");
	dataChanged();
}

void OptionsDlg::removeGCHighlight()
{
	int id =lb_highlightWords->currentItem();
	if(id == -1)
		return;
	opt.gcHighlights.remove(lb_highlightWords->text(id));
	lb_highlightWords->removeItem(id);
	dataChanged();
}

void OptionsDlg::addGCNickColor()
{
	if (le_newNickColor->text().isEmpty())
		return;
	opt.gcNickColors<<le_newNickColor->text();
	lb_nickColors->insertItem(le_newNickColor->text());
	le_newNickColor->setFocus();
	le_newNickColor->setText("");
	displayGCNickColor();
	dataChanged();
}

void OptionsDlg::removeGCNickColor()
{
	int id =lb_nickColors->currentItem();
	if(id == -1)
		return;
	opt.gcNickColors.remove(lb_nickColors->text(id));
	lb_nickColors->removeItem(id);
	dataChanged();
}

void OptionsDlg::chooseGCNickColor()
{
	QColor c = QColorDialog::getColor(c, this);
	if(c.isValid()) {
		QString cs;
		char* t=new char[2];
		sprintf(t,"%02x",c.red());
		cs="#";
		cs=cs+ t;
		sprintf(t,"%02x",c.green());
		cs=cs + t;
		sprintf(t,"%02x",c.blue());
		cs=cs + t;
		delete t;
		le_newNickColor->setText(cs);
		displayGCNickColor();
	}
}

void OptionsDlg::selectedGCNickColor()
{
	int id =lb_nickColors->currentItem();
	if(id == -1)
		return;
	le_newNickColor->setText(lb_nickColors->text(id) );
	displayGCNickColor();
}

void OptionsDlg::displayGCNickColor()
{
	QColor c;
	c.setNamedColor(le_newNickColor->text());
	showColor(pb_nickColor, c);
}
