#include"sslcertdlg.h"

#include<qtextbrowser.h>
#include<qdatetime.h>
#include<qlabel.h>
#include<qpushbutton.h>
#include<qdict.h>
#include"qssl.h"

static void setLabelStatus(QLabel *l, bool ok)
{
	if(ok)
		l->setPaletteForegroundColor(QColor("#2A993B"));
	else
		l->setPaletteForegroundColor(QColor("#810000"));
}

static QString getPropValue(const QValueList<QSSLCertProperty> &list, const QString &var)
{
	QString val;
	for(QValueList<QSSLCertProperty>::ConstIterator it = list.begin(); it != list.end(); ++it) {
		if((*it).var == var) {
			if(!val.isEmpty())
				val += "<br>";
			val += (*it).val;
		}
	}
	return val;
}

static QString makePropEntry(const QString &var, const QString &name, const QValueList<QSSLCertProperty> &list)
{
	QString val = getPropValue(list, var);
	if(val.isEmpty())
		return "";
	else
		return QString("<tr><td><nobr><b>") + name + "</b></nobr></td><td>" + val + "</td></tr>";
}

QString SSLCertDlg::makePropTable(const QString &heading, const QValueList<QSSLCertProperty> &list)
{
	// cat together props of the same name
	QDict<QString> tab;
	for(QValueList<QSSLCertProperty>::ConstIterator it = list.begin(); it != list.end(); ++it) {
		const QSSLCertProperty &p = *it;
		QString *ps = tab.find(p.var);
		if(!ps) {
			ps = new QString;
			tab.insert(p.var, ps);
		}
		QString &s = *ps;

		if(!s.isEmpty())
			s += "<br>";
		s += p.val;
	}

	QString str;
	str += "<tr><td><i>" + heading + "</i><br>";
	str += "<table>";
	str += makePropEntry("O", tr("Organization:"), list);
	str += makePropEntry("OU", tr("Organizational unit:"), list);
	str += makePropEntry("L", tr("Locality:"), list);
	str += makePropEntry("ST", tr("State:"), list);
	str += makePropEntry("C", tr("Country:"), list);
	str += makePropEntry("CN", tr("Common name:"), list);
	str += makePropEntry("Email", tr("Email:"), list);
	str += "</table></td></tr>";
	return str;
}

SSLCertDlg::SSLCertDlg(QWidget *parent, const char *name)
:SSLCertUI(parent, name, true)
{
	setCaption(tr("SSL Cert"));

	connect(pb_close, SIGNAL(clicked()), SLOT(close()));
	pb_close->setDefault(true);
	pb_close->setFocus();
}

void SSLCertDlg::setCert(const QSSLCert &cert)
{
	if(cert.isNull())
		return;

	if(cert.isValid()) {
		lb_valid->setText(QString("The certificate is valid."));
		setLabelStatus(lb_valid, true);
	}
	else {
		lb_valid->setText(QString("The certificate is NOT valid!"));
		setLabelStatus(lb_valid, false);
	}

	QDateTime now = QDateTime::currentDateTime();
	QDateTime notBefore = cert.notBefore();
	QDateTime notAfter = cert.notAfter();
	lb_notBefore->setText(cert.notBefore().toString());
	setLabelStatus(lb_notBefore, now > notBefore);
	lb_notAfter->setText(cert.notAfter().toString());
	setLabelStatus(lb_notAfter, now < notAfter);

	lb_sn->setText(cert.serialNumber());

	QString str;
	str += "<table>";
	str += makePropTable(tr("Subject Details:"), cert.subject());
	str += makePropTable(tr("Issuer Details:"), cert.issuer());
	str += "</table>";
	tb_cert->setText(str);
}

void SSLCertDlg::showCert(const QSSLCert &cert)
{
	SSLCertDlg *w = new SSLCertDlg(0);
	w->setCert(cert);
	w->exec();
	delete w;
}
